// GNU free software repurposed

#include <Arduino.h>
#include "a7105.h"

volatile s16 Channels[NUM_OUT_CHANNELS];

static void initialize() {
    while(1) {
        A7105_Reset();
        if (hubsan_init()) {
          if (verbose) Serial.println("Hubsan_init successful.");
          break;
        }
        else
           if (verbose) Serial.println("Hubsan_init failed.");
    }
    sessionid = rand();
    channel = allowed_ch[6];  // rand() % no_allowed_channels];
}

int hubsan_init()
{
    //set chip ID
    A7105_WriteID(0x55201041);
    //read back chip ID for sanity checking. Works 29/01/14
    //A7105_ReadID();
    
    // 01100011 - enable auto RSSI, auto IF offset, FIFO mode, ADC measurement
    A7105_WriteReg(A7105_01_MODE_CONTROL, 0x63);
    
    // 00001111 - set FIFO length to 16 bytes (easy mode)
    A7105_WriteReg(A7105_03_FIFOI, 0x0f);
    
    // 00000101 - use crystal for timing, set sys clock divider to 2, disable clock generator
    A7105_WriteReg(A7105_0D_CLOCK, 0x05);

    // 00000010 - data rate = Fsyck / 32 / n+1
    A7105_WriteReg(A7105_0E_DATA_RATE, 0x04);
    
    // 00101011 - TX frequency deviation = 186 KHz
    A7105_WriteReg(A7105_15_TX_II, 0x2b);
    
    // 01100010 - disable frequency compensation, disable data invert, bandwidth = 500 KHz, select up-side band
    A7105_WriteReg(A7105_18_RX, 0x62);

    // 10000000 - LNA and mixer gain = 24dB, manual VGA calibrate
    A7105_WriteReg(A7105_19_RX_GAIN_I, 0x80);
    
    // 00001010 - no options here
    A7105_WriteReg(A7105_1C_RX_GAIN_IV, 0x0A);
    
    // 00000111 - reset code register 1. Is this accidental? raise bit 4 to avoid reset and enable crc
    A7105_WriteReg(A7105_1F_CODE_I, 0x07);
    
    // 00010111 - set demodulator default, code error tolerance to 1 bit, preamble pattern detector to 16 bits
    A7105_WriteReg(A7105_20_CODE_II, 0x17);
    
    // 00100111 - demodulator dc level is preamble average val
    A7105_WriteReg(A7105_29_RX_DEM_TEST_I, 0x47);

    // set to standby mode
    A7105_Strobe(A7105_STANDBY);
    
    if (A7105_calibrate_IF() || A7105_calibrate_VCB(0x00) || A7105_calibrate_VCB(0xA0)) {
        if (verbose) Serial.println("Error: calibration failed");
        return 0;
    }

    //Reset VCO Band calibration
    //A7105_WriteReg(0x25, 0x08);

    A7105_SetPower(TXPOWER_150mW);
    if (verbose) Serial.println("Power: Set 150mW");

    A7105_Strobe(A7105_STANDBY);
    return 1;
}

static void update_crc()
{
    int sum = 0;
    for(int i = 0; i < 15; i++)
        sum += packet[i];
    packet[15] = (256 - (sum % 256)) & 0xff;
}



